/**
 * @file wx_human_matting.h
 * */

#ifndef WX_HUMAN_MATTING_H_
#define WX_HUMAN_MATTING_H_

#include <cstdint>
#include <memory>
#include <string>
#include "wx_api.h"
#include "wx_exception.h"
#include "wx_image.h"
#ifdef ANDROID
#include <jni.h>
#endif
namespace wonxai {
class WXHumanMattingImpl;


#ifdef ANDROID
/**
 * @brief 初始化Android版本SDK，并且通过文件方式，为SDK设置license，
 * @details license文件可以使用auth_tools下载或者通过商务索要
 * @note Android版SDK专用
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense,
 * @note 一个进程只需要调用一次，不需要反复调用
 * @param[in] appKey appKey
 * @param[in] licenseFilePath license文件路径，
 * @exception WXException
 */
WX_API void WXSDKInitWithLicense(
        JNIEnv *jniEnv, jobject applicationContext,
        const char *appKey,
        const char *licenseFilePath);

/**
 * @brief 初始化Android版本SDK，并且通过内存数据的方式，为SDK设置license，
 * @details license文件可以使用auth_tools下载或者通过商务索要
 * @note Android版SDK专用
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense,
 * @note 一个进程只需要调用一次，不需要反复调用
 * @param[in] appKey appKey
 * @param[in] licData license数据
 * @param[in] licSize license数据长度
 * @exception WXException
 */
WX_API void WXSDKInitWithLicense(
        JNIEnv *jniEnv, jobject applicationContext,
        const char *appKey,
        uint8_t *licData, size_t licSize);
#else
/**
 * @section WXSDKInitWithLicense
 * @brief 初始化SDK，并且通过文件方式，为SDK设置license，
 * @details license文件可以使用auth_tools下载或者通过商务索要。
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense。
 * @note 一个进程只需要调用一次，不需要反复调用。
 * @note Android系统对应的函数原型为：
 * ```
   void WXSDKInitWithLicense(
        JNIEnv *jniEnv,
        jobject applicationContext,
        const char *appKey,
        const char *licenseFilePath)
 * ```
 * @param[in] appKey appKey
 * @param[in] licenseFilePath license文件路径，
 * @exception WXException
 */
WX_API void WXSDKInitWithLicense(const char *appKey,
                              const char* licenseFilePath);

/**
 * @brief 初始化SDK，并且通过内存数据的方式，为SDK设置license，
 * @details license文件可以使用auth_tools下载或者通过商务索要。
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense。
 * @note 一个进程只需要调用一次，不需要反复调用。
 * @note Android系统对应的函数原型为：
 * ```
   void WXSDKInitWithLicense(
        JNIEnv *jniEnv,
        jobject applicationContext,
        const char *appKey,
        uint8_t *licData,
        size_t licSize)
 * ```
 * @param[in] appKey appKey
 * @param[in] licData license数据
 * @param[in] licSize license数据长度
 * @exception WXException
 */
WX_API void WXSDKInitWithLicense(const char *appKey,
                              uint8_t *licData, size_t licSize);

#endif

/**
 * \section WXSDKTerminate
 * @brief 进程结束前，调用此函数进行资源清理
 * @note 一个进程只需要调用一次，不需要反复调用
 * @note 如果不调动此函数，直接结束进程，有可能在进程退出前发生异常
 */
WX_API void WXSDKTerminate();

/**
 * \section WXSDKVersion
 * @brief 进程结束前，调用此函数进行资源清理
 * @note 一个进程只需要调用一次，不需要反复调用
 * @note 如果不调动此函数，直接结束进程，有可能在进程退出前发生异常
 */
WX_API const char *WXSDKVersion();

/*!
 * \section WXSDKLicense
 * class WXSDKLicense */
class WX_API WXSDKLicense {
public:

    /**
     * @brief 通过文件的方式，为SDK设置license，
     * @details license文件可以使用auth_tools下载或者通过商务索要
     * @note 调用其他api前，必须先调用setLicense函数设置license，
     * @note 一个进程只需要调用一次，不需要反复调用
     * @note 如果是Android系统，只能调用\ref WXSDKInitWithLicense 函数，不能调用此函数。
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKInitWithLicense 函数
     * @param[in]   licensePath: license文件路径，
     * @exception WXException
     */
    static void setLicense(const std::string &licensePath);

    /**
     * @brief 通过内存的方式，为SDK设置license，
     * @details license文件可以使用auth_tools下载或者通过商务索要
     * @note 调用其他api前，必须先调用setLicense函数设置license，
     * @note 一个进程只需要调用一次，不需要反复调用
     * @note 如果是Android系统，只能调用\ref WXSDKInitWithLicense 函数，不能调用此函数。
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKInitWithLicense 函数
     * @param[in]   buffer: license数据，
     * @param[in]   size: license数据长度，
     * @exception WXException
     */
    static void setLicense(uint8_t *buffer, size_t size);

    /**
     * @brief 通过文件的方式，为SDK设置license，
     * @details license文件可以使用auth_tools下载或者通过商务索要
     * @note 调用其他api前，必须先调用setLicense函数设置license，
     * @note 一个进程只需要调用一次，不需要反复调用
     * @note 如果是Android系统，只能调用\ref WXSDKInitWithLicense 函数，不能调用此函数。
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKInitWithLicense 函数
     * @param[in]   licensePath: license文件路径，
     * @param[in]   appKey: appKey，必须与license匹配
     * @exception WXException
     */
    static void setLicense(const char *licensePath, const char *appKey);

    /**
     * @brief 通过内存的方式，为SDK设置license，
     * @details license文件可以使用auth_tools下载或者通过商务索要
     * @note 调用其他api前，必须先调用setLicense函数设置license，
     * @note 一个进程只需要调用一次，不需要反复调用
     * @note 如果是Android系统，只能调用\ref WXSDKInitWithLicense 函数，不能调用此函数。
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKInitWithLicense 函数
     * @param[in]   buffer: license数据
     * @param[in]   size: license数据长度
     * @param[in]   appKey: appKey，必须与license匹配
     * @exception WXException
     */
    static void setLicense(uint8_t *buffer, size_t size, const char *appKey);

    /**
     * @brief 获取当前license文件的有效期起始时间戳
     * @details 单位为秒
     * @note 本函数必须在调用\ref WXSDKInitWithLicense 或者setLicense函数后才能调用，
     * 否则会抛出异常。
     * @return  有效期起始时间戳
     * @exception WXException
     */
    static uint64_t validFrom();

    /**
     * @brief 获取当前license文件的有效期过期时间戳
     * @details 单位为秒
     * @note 本函数必须在调用\ref WXSDKInitWithLicense 或者setLicense函数后才能调用，
     * 否则会抛出异常。
     * @return  有效期起始时间戳
     * @exception WXException
     */
    static uint64_t expiredAt();

    /**
     * @brief 获取当前license文件的licenseId
     * @note 本函数必须在调用\ref WXSDKInitWithLicense 或者setLicense函数后才能调用，
     * 否则会抛出异常。
     * @return  license id
     * @exception WXException
     */
    static const char *licenseId();
private:
    WXSDKLicense(){};
};

/*!
 * @section WXHumanMatting
 * class WXHumanMatting */
class WX_API WXHumanMatting final {
public:
    /**
     * @brief 初始化SDK，调用其他接口（除license相关接口外）之前，需要先调用这个函数。
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKInitWithLicense 函数
     * @note 先调用WXSDKLicense::setLicense函数再调用本函数
     * @note 一个进程只需要调用一次，不需要反复调用
     * @exception WXException
     */
    static void initialize();

    /**
     * @brief 进程结束前，调用此函数进行资源清理
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKTerminate 函数
     * @note 一个进程只需要调用一次，不需要反复调用
     * @note 如果不调动此函数，直接结束进程，有可能在进程退出前发生异常
     */
    static void terminate();

    /**
     * @brief 返回SDK版本号
     * @deprecated: 本函数不再推荐使用，请换成\ref WXSDKVersion 函数
     * @return SDK版本号
     */
    static const char *version();

    /**
     * @brief WXHumanMatting 构造函数
     * @param[in] deviceType 使用GPU还是CPU来创建推理引擎
     * @note 只有证件照服务端版本的SDK才支持GPU版本的推理引擎，其他版本SDK只能使用CPU版本引擎
     * @exception WXException
     */
    WXHumanMatting(WXDeviceType deviceType=kDeviceCPU);

    ~WXHumanMatting();

    /**
     * @brief matting函数，输入rgb像素图，输出alpha图
     * @note 免费版SDK，不能使用本接口，会抛出异常
     * @note 调用之前，outAlpha内存空间由调用者分配，本函数不为它分配内存，
     * 需要确保最少有(width * height * sizeof(float))个字节的存储空间
     * @param[in] inRGB rgb像素图内存块，必须是连续存储的rgb格式，其长度必须是width * height * 3，RGBRGBRGB...
     * @param[in] width inRGB宽度
     * @param[in] height inRGB高度
     * @param[out] outAlpha 输出的0-1.0之间的alpha图，浮点型，宽高和inRGB保持一致，内存由调用者分配，也是连续存储
     * @exception WXException
     */
    void matting(const uint8_t *inRGB,
                 int width,
                 int height,
                 float *outAlpha);

    /**
     * \section removeBackground
     * @brief 图片去背景或换背景
     * @note 免费版SDK，只能换背景，不能去背景（不能返回透明背景图片）
     * @param[in] inImage 输入图，可从\ref WXImage 类创建
     * @param[in] bkImage 背景图，可从\ref WXImage 类创建，
     * <pre>
     * 当本参数设置为nullptr时：
     *     对于付费版SDK，将返回透明背景图,
     *     对于免费版SDK，将返回白色背景图
     * </pre>
     * @return 返回去背景或换背景后的图片
     * @exception WXException
     */
    std::shared_ptr<WXImage> removeBackground(const std::shared_ptr<WXImage> &inImage,
                                              const std::shared_ptr<WXImage> &bkImage);

    /**
     * @brief 图片去背景或换背景
     * @see removeBackground
     */
    WXImage* removeBackground(const WXImage *inImage,
                              const WXImage *bkImage);

private:
    std::unique_ptr<WXHumanMattingImpl> _impl;
};
} //namespace wonxai

#endif //WX_HUMAN_MATTING_H_
