/**
 * @file wx_human_matting_c_api.h
 * SDK初始化和抠图接口
 */
#ifndef WX_HUMAN_MATTING_C_API_H_
#define WX_HUMAN_MATTING_C_API_H_

#ifdef ANDROID
#include <jni.h>
#endif

#include "wx_api.h"
#include "wx_image_c_api.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef ANDROID
WX_API int WXSDKInitWithLicenseFile(
        JNIEnv *jniEnv, jobject applicationContext,
        const char *appKey,
        const char *licenseFilePath);
WX_API int WXSDKInitWithLicenseData(
        JNIEnv *jniEnv, jobject applicationContext,
        const char *appKey,
        uint8_t *licData, size_t licSize);
#else
/**
 * @section WXSDKInitWithLicenseFile
 * @brief 初始化SDK，并且通过文件方式，为SDK设置license，
 * @see wonxai::WXSDKInitWithLicense
 * @details license文件可以使用auth_tools下载或者通过商务索要
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense*函数,
 * @note 一个进程只需要调用一次，不需要反复调用
 * @param[in] appKey appKey
 * @param[in] licenseFilePath license文件路径，
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXSDKInitWithLicenseFile(const char *appKey,
                                    const char* licenseFilePath);

/**
 * @brief 初始化SDK，并且通过内存数据的方式，为SDK设置license，
 * @see wonxai::WXSDKInitWithLicense
 * @details license文件可以使用auth_tools下载或者通过商务索要
 * @note 调用其他api前，必须先调用WXSDKInitWithLicense*函数,
 * @note 一个进程只需要调用一次，不需要反复调用
 * @param[in] appKey appKey
 * @param[in] licData license数据
 * @param[in] licSize license数据长度
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXSDKInitWithLicenseData(const char *appKey,
                                    uint8_t *licData, size_t licSize);
#endif

/**
 * @brief 程序退出前，调用本函数释放资源 */
WX_API void WXSDK_Terminate();

/**
 * @brief 获取sdk版本号
 */
WX_API const char *WXSDK_Version();

/**
 * @brief 获取当前license文件的有效期起始时间戳，单位为秒 */
WX_API int WXSDKLicense_ValidFrom(uint64_t *validFrom);

/**
 * @brief 获取当前license文件的有效期过期时间戳，单位为秒 */
WX_API int WXSDKLicense_ExpiredAt(uint64_t *expiredAt);

/**
 * @brief 获取当前license文件的licenseId */
WX_API const char * WXSDKLicense_LicenseId();

/**
 * @section WXHumanMatting_t
 */

/** WXHumanMatting_t结构对象 */
typedef struct WXHumanMatting_t WXHumanMatting_t;

/**
 * 创建WXHumanMatting_t对象
 * @see wonxai::WXHumanMatting::WXHumanMatting
 * @param[in] deviceType 是否使用gpu
 * @param[out] matting_p 创建并返回WXHumanMatting_t对象
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXHumanMatting_Create(WXDeviceType deviceType, WXHumanMatting_t **matting_p);

/** 释放WXHumanMatting_t对象 */
WX_API void WXHumanMatting_Release(WXHumanMatting_t *matting);

/**
 * @brief matting函数，输入rgb像素图，输出alpha图
 * @see wonxai::WXHumanMatting::matting
 * @note 免费版SDK，不能使用本接口，调用的话，会返回错误
 * @note 调用之前，outAlpha内存空间由调用者分配，本函数不为它分配内存，
 * 需要确保最少有(width * height * sizeof(float))个字节的存储空间
 * @param[in] matting WXHumanMatting_t对象
 * @param[in] inRGB rgb像素图内存块，必须是连续存储的rgb格式，其长度必须是width * height * 3，RGBRGBRGB...
 * @param[in] width inRGB宽度
 * @param[in] height inRGB高度
 * @param[out] outAlpha 输出的0-1.0之间的alpha图，浮点型，宽高和inRGB保持一致，内存由调用者分配，也是连续存储
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 * */
WX_API int WXHumanMatting_Matting(WXHumanMatting_t *matting,
                                  const uint8_t *inRGB,
                                  int width,
                                  int height,
                                  float *outAlpha);

/**
 * @brief 图片去背景或换背景
 * @see wonxai::WXHumanMatting::removeBackground
 * @note 免费版SDK，只能换背景，不能去背景（不能返回透明背景图片）
 * @param[in] matting WXHumanMatting_t对象
 * @param[in] inImage 输入图，\ref WXImage_t
 * @param[in] bkImage 背景图，\ref WXImage_t
 * <pre>
 * 当本参数设置为nullptr时：
 *     对于付费版SDK，将返回透明背景图,
 *     对于免费版SDK，将返回白色背景图
 * </pre>
 * @param[out] resultImg_p 返回去背景或换背景后的图片
 * @return 返回去背景或换背景后的图片
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXHumanMatting_RemoveBackground(WXHumanMatting_t *matting,
                                           const WXImage_t *inImage,
                                           const WXImage_t *bkImage,
                                           WXImage_t **resultImg_p);

/**
 * 返回SDK C接口的最近一条错误描述
 * @note 本函数是个线程相关函数，它只返回本线程的最近一条错误
 * @note 本函数只用户返回C接口的错误，不能返回C++接口的错误 */
WX_API const char *WXSDK_GetLastErrorMsg();

#ifdef __cplusplus
}
#endif

#endif // WX_HUMAN_MATTING_C_API_H_
