/**
 * @file wx_image.h
 * SDK图片操作类
 */

#ifndef WX_IMAGE_H_
#define WX_IMAGE_H_
#include <memory>
#include <string>
#include "wx_api.h"

namespace wonxai {

/*! class WXImage */
class WX_API WXImage {
public:
    /*! 图像数据格式 */
    enum DataFormat{
        /** 未知格式 */
        UNK = 0,

        /** 三通道RGB格式 */
        RGB = 1,

        /** 四通道RGBA格式 */
        RGBA,

        /** 单通道ALPHA格式 */
        ALPHA8,

        /** 单通道浮点值 */
        FLOAT32
    };

    /**
     * 从图片文件创建WXImage对象，支持jpg和png图片
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromFile(const std::string &fileName);

    /**
     * 从图片原数据（即图片编码数据）创建WXImage对象，支持jpg和png图片
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromEncodedData(const uint8_t *data, size_t size);

    /**
     * 创建一副纯色图，颜色值由r，g，b三个值指定
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromColor(int width, int height,
                                              uint8_t r, uint8_t g, uint8_t b);

    /**
     * 根据给定的图像像素数据，创建WXImage对象，数据格式必须为RGB三通道，且连续存储
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromBuffer(int width, int height,
                                               const uint8_t *rgbData);

    /**
     * 根据给定的图像像素数据，创建WXImage对象，数据格式必须为RGBA四通道，且连续存储
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromRGBAData(int width, int height,
                                               const uint8_t *rgbaData);

    /**
     * 从另外一个WXImage对象复制一个新的WXImage对象
     * @exception WXException
     */
    static std::shared_ptr<WXImage> fromImage(WXImage *other);

    /**
     * @see fromFile
     */
    static WXImage* fromFile2(const std::string &fileName);

    /**
     * @see fromEncodedData
     */
    static WXImage* fromEncodedData2(const uint8_t *data, size_t size);

    /**
     * @see fromColor
     */
    static WXImage* fromColor2(int width, int height,
                                              uint8_t r, uint8_t g, uint8_t b);

    /**
     * @see fromBuffer
     */
    static WXImage* fromBuffer2(int width, int height,
                                               const uint8_t *rgbData);

    /**
     * @see fromRGBAData
     */
    static WXImage* fromRGBAData2(int width, int height,
                                  const uint8_t *rgbaData);

    virtual ~WXImage();

    /**
     * 将图片保存到文件，支持jpg和png图片
     * param[in] fileName 目标文件路径，扩展名必须为".jpg"或者".png"（不区分大小写）
     * @exception WXException
     */
    virtual void toFile(const std::string &fileName);

    /**
     * 将图片保存为编码后的数据，支持jpg和png格式
     * param[in] encodedData  用于接收编码后的二进制数据
     * param[in] imgType  图片编码类型：jpg or png
     * @exception WXException
     */
    virtual void toEncodedData(std::string &encodedData,
                               const std::string &imgType /* "jpg" or "png" */);

    /** 返回图片宽度 */
    virtual int width();

    /** 返回图片高度 */
    virtual int height();

    /** 返回图片数据，调用者不能从外面释放 */
    virtual uint8_t *data();

    /** 返回图片数据格式 \ref DataFormat */
    virtual DataFormat dataFormat();

protected:
    WXImage();

};

}

#endif //WX_IMAGE_H_
